# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import typing
import types
import inspect

from tcutils.const import CLASS_METHOD_TYPES

def class_methods(
    cls: typing.Any,
    method_types: typing.Iterable[typing.Type] = CLASS_METHOD_TYPES
) -> typing.List[str]:
    """Return list of methods from the class.
    """
    return [name for name, kind in cls.__dict__.items() \
            if type(kind) in method_types]


def class_prefixed_methods(cls: typing.Any, prefix: str) -> typing.List[str]:
    """Return list of methods in a class that start with prefix.
    """
    methods = class_methods(cls)
    return list(filter(lambda x: x.startswith(prefix), methods))


def module_classes(
    module: types.ModuleType,
    class_type: typing.Optional[str] = None
):
    """Return list of module classes.

    Optionally filter by `class_type`.
    """
    class_list = [item[1] for item in inspect.getmembers(module, inspect.isclass)]
    if class_type:
        class_list = [cls for cls in class_list if issubclass(cls, class_type)]
    return class_list



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pytest
import yaml
import pathlib

from ..context import tcutils

BASE_YAML_PATH = 'data/base.yaml'

FULL_YAML_DICT = {
    'base': {
        'somevar': 'test',
        'somelist': ['one', 'two'],
        'included': {
            'other': {
                'foo': 'bar'
            }
        }
    }
}


class TestYamlInclude:

    @pytest.fixture
    def base_yaml_stream(self):
        yaml_path = pathlib.Path(__file__).parent.absolute() / BASE_YAML_PATH
        return yaml_path.open('r')

    def test_yaml_include(self, base_yaml_stream):
        contents = yaml.load(base_yaml_stream,
            tcutils.yamlinclude.IncludeLoader)
        assert contents == FULL_YAML_DICT



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import string
import types


VALID_FILENAME_CHARS = "-_.() %s%s" % (string.ascii_letters, string.digits)
DEFAULT_REPLACEMENT_CHAR = '_'

CLASS_METHOD_TYPES = [types.FunctionType, classmethod, staticmethod]

DEFAULT_URI_SCHEME = 'file'

DEFAULT_ADAPTER_METHOD = 'execute'



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import click

import types
from tcutils.types import ClickGroupOrCommand


class NotRequiredIf(click.Option):
    def __init__(self, *args, **kwargs):
        self.not_required_if = kwargs.pop('not_required_if')
        assert self.not_required_if, "'not_required_if' parameter required"
        kwargs['help'] = (kwargs.get('help', '') +
            ' NOTE: This argument is mutually exclusive with %s' %
            self.not_required_if
        ).strip()
        super(NotRequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        we_are_present = self.name in opts
        other_present = self.not_required_if in opts

        if other_present:
            if we_are_present:
                raise click.UsageError(
                    "Illegal usage: `%s` is mutually exclusive with `%s`" % (
                        self.name, self.not_required_if))
            else:
                self.prompt = None

        return super(NotRequiredIf, self).handle_parse_result(
            ctx, opts, args)


def cli_add_groups(
    parent: ClickGroupOrCommand,
    group_module: types.ModuleType
):
    """Attach Click sub groups to Click parent."""
    groups = [
        group_function
        for group_function in vars(group_module).values()
        if isinstance(group_function, click.Group)
    ]
    parent_add_group = getattr(parent, 'add_command')
    for group_function in groups:
        parent_add_group(group_function)


def cli_add_commands(
    parent: ClickGroupOrCommand,
    command_module: types.ModuleType
):
    """Attach Click sub commands to Click parent."""
    commands = [
        command_function
        for command_function in vars(command_module).values()
        if isinstance(command_function, click.Command)
    ]

    parent_add_command = getattr(parent, 'add_command')
    for command_function in commands:
        parent_add_command(command_function)


def cli_register_command_group(manager, cli_parent):
    manager.scan()
    for adapter_name in manager:
        adapter = manager.get(adapter_name)
        commands = adapter.cli_commands()
        cli_group_kwargs = adapter.cli_group_kwargs
        if not cli_group_kwargs:
            cli_group_kwargs = {}
        cli_group = click.Group(
            name=adapter.cli_group,
            **cli_group_kwargs
        )
        for command in commands:
            cli_group.add_command(command)
        cli_parent.add_command(cli_group)



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pathlib
import os
import sys
import typing
import tempfile
from tcutils.types import UniversalPath, UniversalPathCollection, \
    KeywordArgsType


if sys.platform == 'win32':
    DEFAULT_EXPANDVARS = pathlib.ntpath.expandvars
else:
    DEFAULT_EXPANDVARS = pathlib.posixpath.expandvars


# Path functions


def normalize_path(
   path: UniversalPath,
   use_default_expansion: bool = False,
   default_expandvars: typing.Callable = DEFAULT_EXPANDVARS
) -> pathlib.Path:
    """Normalize path. If use_default_expansion is True,
    default_expandvars will be used as variable expansion.
    """
    temp_path = pathlib.Path(path)
    if use_default_expansion:
        temp_path = default_expandvars(temp_path)
    else:
        try:
            temp_path = temp_path.expandvars(temp_path)
        except:
            temp_path = default_expandvars(temp_path)
    temp_path = pathlib.Path(temp_path)
    temp_path = temp_path.expanduser()
    return temp_path.resolve()


def get_path(
    path: UniversalPath,
    **normalize_kwargs: KeywordArgsType,
) -> pathlib.Path:
    """Alias for normalize_path.
    """
    return normalize_path(path, **normalize_kwargs)


def check_path(
    path: UniversalPath,
    **normalize_kwargs: KeywordArgsType,
) -> pathlib.Path:
    """Checks path and return normalized Path.
    """
    normalized_path = normalize_path(path, **normalize_kwargs)
    if not normalized_path.exists():
        raise IOError(f'Path "{path}" does not exist')
    return normalized_path


def join_paths(
    *paths: UniversalPathCollection,
    **normalize_kwargs: KeywordArgsType,
) -> pathlib.Path:
    """Joins iterable of paths to one path.
    """
    joined_path = normalize_path(paths[0], **normalize_kwargs)
    for path_part in paths[1:]:
        joined_path = joined_path.joinpath(path_part)
    joined_path = normalize_path(joined_path, **normalize_kwargs)
    return joined_path


def make_relative_path(path: UniversalPath) -> pathlib.Path:
    """Strip leading / from path to make it relative.
    """
    return pathlib.Path(str(path)[1:]) if str(path).startswith('/') else path


# Directory functions


def current_dir(
    **normalize_kwargs: KeywordArgsType,
) -> pathlib.Path:
    """Return current directory Path.
    """
    current_dir_path = normalize_path(pathlib.Path.cwd(), **normalize_kwargs)
    return current_dir_path


def create_dirs(
    dirs_to_create: UniversalPathCollection,
    parent_dir: typing.Optional[UniversalPath] = None,
    create_parent: bool = False,
    **normalize_kwargs: KeywordArgsType,
) -> typing.List[pathlib.Path]:
    """Create directories.

    If `parent_dir` is specified it will create directories from the list
    as subfolders of `parent_dir`.

    If `create_parent` is `False`, and
    `parent_dir` path does not exist function will `raise IOError`, otherwise
    `parent_dir` will be created.
    """
    parent_path = normalize_path(parent_dir, **normalize_kwargs) if parent_dir \
        else current_dir()

    created_paths = []

    if not parent_path.exists():
        if create_parent:
            parent_path.mkdir(parents=True, exist_ok=True)
        else:
            raise IOError(f'Path "{parent_path}" does not exist')
    if create_parent:
        created_paths.append(parent_path)

    for target_dir in dirs_to_create:
        target_path = parent_path.joinpath(pathlib.Path(target_dir)).resolve()
        if not target_path.exists():
            target_path.mkdir(parents=True, exist_ok=True)
        created_paths.append(target_path)

    return created_paths


def temp_system_dir_path():
    """Return temporary system directory path.
    """
    temp_file = tempfile.TemporaryFile()
    return pathlib.Path(tempfile.tempdir)


def list_dir(path: UniversalPath, recursive: bool=False):
    """List directory contents.
    """
    path = check_path(path)
    glob_pattern = '**/*' if recursive else '*'
    return path.glob(glob_pattern)


# File functions


def check_file_exists_or_increment(
    filepath: UniversalPath,
    **normalize_kwargs: KeywordArgsType
) -> pathlib.Path:
    """Return file path with incremented suffix, if `filepath` already exists.
    Will search for first available number suffix, if previous ones are already
    existing, e.g. `FILENAME_1.EXTENSION`.
    """
    number = 1
    temp_path = normalize_path(filepath, **normalize_kwargs)
    if temp_path.exists():
        filepath_ext = temp_path.suffix
        if filepath_ext:
            filepath_root = str(temp_path)[:-len(str(filepath_ext))]
        else:
            filepath_root = str(temp_path)
        # FIXME: This is inefficient, should list all files and choose
        #        correct number
        while 1:
            new_filepath_root = filepath_root + '_' + str(number)
            new_filepath = pathlib.Path(new_filepath_root + filepath_ext)
            if new_filepath.exists():
                number += 1
                continue
            else:
                filepath = new_filepath
                break
    return filepath



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pytest
import pathlib
import sys

from ..context import tcutils

FS_CLEAN_FILENAMES = [
    ('normal_file.txt', tcutils.const.VALID_FILENAME_CHARS, ' ',
    tcutils.const.DEFAULT_REPLACEMENT_CHAR, 'normal_file.txt'),
    ('spaced file.txt', tcutils.const.VALID_FILENAME_CHARS, ' ',
    tcutils.const.DEFAULT_REPLACEMENT_CHAR, 'spaced_file.txt'),
    ('On€ Unicódę File', tcutils.const.VALID_FILENAME_CHARS, ' ',
    tcutils.const.DEFAULT_REPLACEMENT_CHAR, 'On_Unicode_File'),
    ('abc(12).tar.gz', tcutils.const.VALID_FILENAME_CHARS, ' ',
    tcutils.const.DEFAULT_REPLACEMENT_CHAR, 'abc(12).tar.gz'),
    ('test-file@ bla.!something', tcutils.const.VALID_FILENAME_CHARS, ' @!',
    tcutils.const.DEFAULT_REPLACEMENT_CHAR, 'test-file__bla._something'),
]

POSIX_PERM_TRIAD_TEST = [
    (False, False, False, 0o0),
    (False, False, True, 0o1),
    (False, True, False, 0o2),
    (True, False, False, 0o4),
    (False, True, True, 0o3),
    (True, False, True, 0o5),
    (True, True, False, 0o6),
    (True, True, True, 0o7),
]
POSIX_PERM_TRIAD_TEST_STR = [
    ('0', 0o0),
    ('1', 0o1),
    ('2', 0o2),
    ('4', 0o4),
    ('3', 0o3),
    ('5', 0o5),
    ('6', 0o6),
    ('7', 0o7),
]
POSIX_PERM_TEST_STR = [
    ('000', '0000'),
    ('100', '0100'),
    ('010', '0010'),
    ('001', '0001'),
    ('775', '0775'),
    ('644', '0644'),
    ('1775', '1775'),
    ('640', '0640'),
    ('700', '0700'),
    ('1777', '1777'),
]
POSIX_PERM_TEST_STAT = [
    (33152, '0600'),
    (33700, '1644'),
    (32876, '0154'),
]
POSIX_PERM_TEST_OCTET = [
    '0600',
    '0640',
    '0644',
    '0750',
    '1755',
]

class TestFS:

    @pytest.fixture
    def default_expandvars(self):
        return pathlib.ntpath.expandvars if sys.platform == 'win32' else \
            pathlib.posixpath.expandvars

    @pytest.mark.parametrize(
        "read, write, execute, octet", POSIX_PERM_TRIAD_TEST
    )
    def test_posix_perm_triad(self, read, write, execute, octet):
        triad = tcutils.fs.PosixPermissionTriad(
            read=read, write=write, execute=execute)
        assert octet == triad.to_octal()

    @pytest.mark.parametrize(
        "input_str, octet", POSIX_PERM_TRIAD_TEST_STR
    )
    def test_posix_perm_triad_from_str(self, input_str, octet):
        triad = tcutils.fs.PosixPermissionTriad.from_octal(input_str)
        assert octet == triad.to_octal()

    @pytest.mark.parametrize(
        "input_str, octet", POSIX_PERM_TEST_STR
    )
    def test_posix_perm(self, input_str, octet):
        perm = tcutils.fs.PosixPermissions.from_octal(input_str)
        assert octet == perm.to_octal_str()

    @pytest.mark.parametrize(
        "input_str, octet", POSIX_PERM_TEST_STR
    )
    def test_posix_perm_to_octal(self, input_str, octet):
        perm = tcutils.fs.PosixPermissions.from_octal(input_str)
        assert int(octet, 8) == perm.to_octal()

    @pytest.mark.parametrize(
        "stat_mode, octet", POSIX_PERM_TEST_STAT
    )
    def test_posix_perm_stat(self, stat_mode, octet):
        perm = tcutils.fs.PosixPermissions.from_stat(stat_mode)
        assert octet == perm.to_octal_str()

    @pytest.mark.parametrize(
        "octet", POSIX_PERM_TEST_OCTET
    )
    def test_posix_perm_apply_file(self, octet):
        temp_file = tcutils.fs.temp_file()
        temp_file_path = pathlib.Path(temp_file.name)
        perm = tcutils.fs.PosixPermissions.from_octal(octet)
        perm.apply(temp_file_path)
        temp_file_perms = oct(temp_file_path.stat().st_mode)[-4:]
        assert temp_file_perms == perm.to_octal_str()

    @pytest.mark.parametrize(
        "filename, whitelist, replace, replacement, result", FS_CLEAN_FILENAMES
    )
    def test_clean_filename(self, filename, whitelist, replace, replacement,
        result
    ):
        cleaned_filename = tcutils.fs.clean_filename(
            filename, whitelist, replace, replacement)
        assert cleaned_filename == result



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pytest
import pathlib
import sys

from ..context import tcutils


current_user_name = tcutils.user.current_user_name()

POSIX_PATHS = [
    ("/home/someuser", "/home/someuser", pathlib.PosixPath),
    ("$HOME", "/home/" + current_user_name, pathlib.PosixPath),
    ("/usr/lib/python/../../bin/python", "/usr/bin/python", pathlib.PosixPath),
]

@pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
class TestPosixPaths:

    @pytest.fixture
    def default_expandvars(self):
        return pathlib.posixpath.expandvars

    @pytest.mark.parametrize(
        "path, result, path_type", POSIX_PATHS
    )
    @pytest.mark.parametrize(
        "use_default_expansion", [True, False]
    )
    def test_normalize_path(self, path, result, path_type,
        use_default_expansion, default_expandvars
    ):
        normalized_path = tcutils.paths.normalize_path(path,
            use_default_expansion, default_expandvars)
        assert normalized_path == path_type(result)

    def test_temp_system_dir_path(self):
        temp_dir = tcutils.paths.temp_system_dir_path()
        assert pathlib.Path('/tmp') == temp_dir



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

# Cannot be used with Python3.6
#from __future__ import annotations

import yaml
import logging
import typing

from marshmallow import Schema
from marshmallow.exceptions import ValidationError

from tcutils.types import UniversalPath
from tcutils.paths import check_path
from tcutils.funcutils import class_prefixed_methods
from tcutils.yamlinclude import IncludeLoader

log = logging.getLogger(__file__)


class BaseConfigSchema(Schema):
    """Placeholder base configuration schema.

    You can use it as a base for your own schema by inheriting this class.
    """
    pass


class ConfigurationAttribute:
    """Configuration attribute for easy access."""

    def __init__(self, value: typing.Any):
        self.value = value

    def __str__(self) -> str:
        return str(self.value)

    def __getattr__(self, attr_name) -> typing.Any:
        attr_suffix = None
        if attr_name.find('.') > -1:
            attr_name, attr_suffix = attr_name.split('.', 1)
        if attr_name not in dir(self):
            if attr_name not in object.__getattribute__(self, 'value'):
                raise AttributeError(f'No such attribute: {attr_name}')
            attr_value = getattr(self, 'value').get(attr_name)
            if type(attr_value) is dict:
                attr_value = ConfigurationAttribute(attr_value)
        else:
            attr_value = super().__getattribute__(attr_name)
        if attr_suffix:
            attr_value = getattr(attr_value, attr_suffix)
        return attr_value


class Configuration:
    """Configuration class."""

    def __init__(
        self,
        config_dict: typing.Dict[str, typing.Any],
        schema: BaseConfigSchema = BaseConfigSchema
    ):
        self.config = config_dict
        self.schema = schema
        self._parse()

    def __str__(self) -> str:
        return f'Configuration({self.config})'

    def __getitem__(self, k: str) -> typing.Any:
        return self.config.get(k)

    def __getattr__(self, attr_name: str) -> typing.Any:
        attr_suffix = None
        if attr_name.find('.') > -1:
            attr_name, attr_suffix = attr_name.split('.', 1)
        if attr_name not in dir(self):
            if attr_name not in object.__getattribute__(self, 'config'):
                raise AttributeError(f'No such attribute: {attr_name}')
            attr_value = getattr(self, 'config').get(attr_name)
            if type(attr_value) is dict:
                attr_value = ConfigurationAttribute(attr_value)
        else:
            attr_value = super().__getattribute__(attr_name)
        if attr_suffix:
            attr_value = getattr(attr_value, attr_suffix)
        return attr_value

    @classmethod
    def load(
        cls: typing.Type[typing.Any],
        config_path: UniversalPath,
        schema_class: typing.Optional[BaseConfigSchema] = None,
        *schema_args: typing.Iterable[typing.Any],
        **schema_kwargs: typing.Mapping[typing.Any, typing.Any],
    ) -> typing.Type[typing.Any]:
        """Load configuration file."""
        p = check_path(config_path)
        with p.open('r') as fp:
            try:
                config_dict = yaml.load(fp, IncludeLoader)
            except FileNotFoundError as e:
                log.error(f'Configuration !include error: {e}')
                raise e
        if schema_class is None:
            schema_class = BaseConfigSchema
        schema = schema_class(*schema_args, **schema_kwargs)
        try:
            result = schema.load(config_dict)
        except ValidationError as e:
            log.error(f'Configuration file error: "{e.args}"')
            raise e
        if schema_class:
            return cls(result, schema=schema_class)
        else:
            return cls(result)

    def _parse(self):
        """Parse configuration and trigger events if necessary."""
        parsing_methods = class_prefixed_methods(self.__class__, '_parse_')
        for parsing_method in parsing_methods:
            method = getattr(self, parsing_method)
            method()



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import logging
import os
import sys
import io
import unicodedata
import pathlib
import typing
import tempfile
import urllib.request
import urllib.response
from dataclasses import dataclass
from tcutils.const import VALID_FILENAME_CHARS, DEFAULT_REPLACEMENT_CHAR, \
    DEFAULT_URI_SCHEME
from tcutils.types import CharsList, UniversalPath, KeywordArgsType
from tcutils.paths import normalize_path

log = logging.getLogger(__file__)


@dataclass
class PosixPermissionTriad:
    read: bool
    write: bool
    execute: bool

    @classmethod
    def from_octal(cls, octet: typing.Union[int, str]):
        if type(octet) == str:
            octet = int(octet, 8)
        return cls(
            read = octet & 4,
            write = octet & 2,
            execute = octet & 1
        )

    def to_octal(self) -> int:
        octet = 0x0
        if self.read:
            octet += 0x4
        if self.write:
            octet += 0x2
        if self.execute:
            octet += 0x1
        return octet


@dataclass
class PosixPermissions:
    owner: PosixPermissionTriad
    group: PosixPermissionTriad
    world: PosixPermissionTriad
    sticky: bool = False

    @classmethod
    def from_octal(cls, perm_octet: typing.Union[str, int]):
        sticky = False
        if type(perm_octet) == int:
            perm_octet = str(perm_octet)
        if len(perm_octet) > 4 or len(perm_octet) < 3:
            raise ValueError('Permissions must be in POSIX octal format.')
        elif len(perm_octet) == 4:
            sticky = int(perm_octet[0], 8) & 1
            perm_octet = perm_octet[1:]
        return cls(
            owner = PosixPermissionTriad.from_octal(perm_octet[0]),
            group = PosixPermissionTriad.from_octal(perm_octet[1]),
            world = PosixPermissionTriad.from_octal(perm_octet[2]),
            sticky = sticky
        )

    @classmethod
    def from_stat(cls, stat_mode: int):
        perm_octet = oct(stat_mode)[-4:]
        return cls.from_octal(perm_octet)

    def to_octal_str(self):
        octet_list = [
            1 if self.sticky else 0,
            self.owner.to_octal(),
            self.group.to_octal(),
            self.world.to_octal(),
        ]
        return ''.join([str(octet) for octet in octet_list])

    def to_octal(self):
        return int(self.to_octal_str(), 8)

    def apply(self, path: UniversalPath):
        """Apply permissions to path."""
        if type(path) == str:
            path = pathlib.Path(path)
        path.chmod(self.to_octal())

    def __str__(self):
        return self.to_octal_str()


def clean_filename(
    filename: str,
    whitelist: CharsList = VALID_FILENAME_CHARS,
    replace: CharsList = ' ',
    replacement: str = DEFAULT_REPLACEMENT_CHAR
) -> str:
    """Cleans filename according to `whitelist` characters and `replace`
    characters which will be substituted by `replacement` character.
    """
    # Replace spaces (or characters defined by replace)
    for r in replace:
        filename = filename.replace(r, replacement)

    # Keep only valid ASCII chars
    cleaned_filename = unicodedata.normalize(
        'NFKD', filename).encode('ASCII', 'ignore').decode()

    # Keep only whitelisted chars
    return ''.join(c for c in cleaned_filename if c in whitelist)


def temp_dir() -> tempfile.TemporaryDirectory:
    """Return temporary directory."""
    return tempfile.TemporaryDirectory()


def temp_file() -> tempfile.NamedTemporaryFile:
    """Return temporary file."""
    return tempfile.NamedTemporaryFile()


def open_uri(
    uri: str,
    default_uri_scheme: str=DEFAULT_URI_SCHEME,
    default_input_stream: io.StringIO=sys.stdin,
    *args, **kwargs
) -> typing.Union[urllib.response.addinfourl, io.StringIO]:
    """Open URI and return stream handle."""
    if uri == '-':
        # Reading from stream
        if not hasattr(default_input_stream, 'name'):
            default_input_stream.name = '<stream>'
        return default_input_stream
    # Reading from URI
    parsed_uri = urllib.request.urlparse(uri)
    uri_scheme = parsed_uri.scheme
    if uri_scheme == '' or len(uri_scheme) == 1:
        # Set drive path if it's local Windows path
        uri_drive_path = uri_scheme if len(uri_scheme) == 1 else ''
        parsed_uri = parsed_uri._replace(scheme=default_uri_scheme)
        if default_uri_scheme == 'file':
            if uri_drive_path:
                uri_path = str(
                    (pathlib.Path(uri_drive_path) / parsed_uri.path).resolve())
                parsed_uri = parsed_uri._replace(path=uri_path)
            else:
                uri_path = parsed_uri.path
                if not uri_path.startswith('/'):
                    uri_path = str(pathlib.Path().cwd() / uri_path)
                    parsed_uri = parsed_uri._replace(path=uri_path)
    return urllib.request.urlopen(parsed_uri.geturl(), *args, **kwargs)



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import typing
from dataclasses import dataclass, field

from ..context import tcutils


@dataclass
class SomeAdapter(tcutils.adapters.BaseAdapter):
    name: str = "SomeAdapter"

    def execute(self, *args, **kwargs):
        return f'{self.name} executing'


@dataclass
class SomeAdapterManager(tcutils.adapters.BaseAdapterManager):
    name: str = "SomeAdapterManager"
    namespaces: tcutils.types.AdapterManagerNamespaces = field(
        default_factory=lambda: ['tcutils.tests.adapters.test_adapters'])
    adapter_class: typing.Type[tcutils.adapters.BaseAdapter] = SomeAdapter



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

try:
    import pwd
except ModuleNotFoundError:
    import getpass
import os


def current_user_name() -> str:
    """Return current user name."""
    try:
        return pwd.getpwuid(os.getuid())[0]
    except NameError:
        return getpass.getuser()



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import abc
import typing
import sys
import click
from dataclasses import dataclass, field
from functools import update_wrapper
from click.globals import get_current_context

from tcutils.const import DEFAULT_ADAPTER_METHOD
from tcutils.types import AdapterNameOrClass, AdapterManagerNamespaces
from tcutils.funcutils import module_classes


def adapter_command(name=None, cls=None, **attrs):
    """Decorator for adding Click command within Adapter.
    """
    if cls is None:
        cls = click.Command

    def decorator(func):
        func._adapter_command = True
        func._adapter_command_name = name
        func._adapter_command_cls = cls
        func._adapter_command_attrs = attrs
        return func
    return decorator


def adapter_pass_context(f):
    """Decorator for passing click context as second argument for Adapter.
    """

    def new_func(*args, **kwargs):
        if args:
            args = args[:1] + (get_current_context(),) + args[1:]
        return f(*args, **kwargs)

    return update_wrapper(new_func, f)


@dataclass
class BaseAdapter(abc.ABC):
    name: str

    @abc.abstractmethod
    def execute(self, *args, **kwargs):
        raise NotImplementedError


@dataclass
class BaseCliAdapter(BaseAdapter):
    cli_group: str

    def _get_command_functions(self):
        items = [item for item in dir(self)]
        methods = list(map(lambda x: getattr(self, x), items))
        return [item for item in methods
            if hasattr(item, '_adapter_command')
            and getattr(item, '_adapter_command')]

    def cli_commands(self):
        commands = []
        for f in self._get_command_functions():
            name = f._adapter_command_name
            attrs = f._adapter_command_attrs
            cls = f._adapter_command_cls
            params = f.__click_params__ if hasattr(f, '__click_params__') else []
            cmd = click.decorators._make_command(f, name, attrs, cls)
            # Not sure why is this needed
            cmd.params = params
            cmd.__doc__ = f.__doc__
            commands.append(cmd)
        return commands


@dataclass
class BaseAdapterManager(abc.ABC):
    """AdapterManager abstract base class.
    """
    name: str
    namespaces: AdapterManagerNamespaces
    adapter_class: typing.Type[BaseAdapter]
    adapter_dict: typing.Dict[str, BaseAdapter] = field(
        default_factory=dict)

    @staticmethod
    def _get_adapter_name(adapter_name_or_class: AdapterNameOrClass):
        """Return adapter name from either name or adapter class itself.
        """
        if type(adapter_name_or_class) is not str:
            if not issubclass(adapter_name_or_class, BaseAdapter):
                raise ValueError(
                    f'"{adapter_name_or_class}" is not BaseAdapter type.')
            adapter_name = adapter_name_or_class.name
        else:
            adapter_name = adapter_name_or_class
        return adapter_name

    def get(self,
        adapter_name_or_class: AdapterNameOrClass
    ):
        """Return adapter."""
        adapter_name = self._get_adapter_name(adapter_name_or_class)
        if adapter_name not in self.adapter_dict:
            raise ValueError(f'Adapter "{adapter_name}" is not registered.')
        return self.adapter_dict.get(adapter_name)

    def list(self, objects=False):
        """Return list of registered adapters.

        If objects=True return Adapter instances
        """
        if objects:
            result = []
            for item in self.adapter_dict.values():
                if type(item) is list:
                    result.extend(item)
                else:
                    result.append(item)
            return result
        else:
            return list(self.adapter_dict.keys())

    def __iter__(self):
        """Iterate over registered adapters.
        """
        for adapter_name in self.adapter_dict:
            yield adapter_name

    def execute(self,
        adapter_name_or_class: AdapterNameOrClass,
        method: typing.Optional[str] = None,
        *args,
        **kwargs
    ):
        """Execute adapter method (default: BaseAdapter.execute()).
        """
        adapter_name = self._get_adapter_name(adapter_name_or_class)
        adapter_object = self.get(adapter_name)

        if method:
            if not hasattr(adapter_object, method):
                raise ValueError(
                    f'`{adapter_object}.{method}` does not exist.')
            adapter_method = getattr(adapter_object, method)
        else:
            adapter_method = getattr(adapter_object, DEFAULT_ADAPTER_METHOD)

        if not callable(adapter_method):
            raise ValueError(
                f'`{adapter_object}.{method}` is not callable.')

        return adapter_method(*args, **kwargs)

    def register(self,
        adapter_class: typing.Type[BaseAdapter],
        allow_multiple: bool=False,
        allow_substitute: bool=False,
        *args, **kwargs
    ):
        """Register `adapter_class` in manager.
        """
        adapter_name = adapter_class.name
        if allow_multiple and adapter_name in self.adapter_dict:
            if type(self.adapter_dict[adapter_name]) is not list:
                self.adapter_dict[adapter_name] = [
                    self.adapter_dict[adapter_name]]
            self.adapter_dict[adapter_name].append(
                adapter_class(*args, **kwargs))
        else:
            if not allow_substitute and adapter_name in self.adapter_dict:
                raise RuntimeError(
                    f'Adapter {adapter_class} already registered')
            self.adapter_dict[adapter_name] = adapter_class(*args, **kwargs)

    def remove(self,
        adapter_name_or_class: AdapterNameOrClass
    ):
        """Remove registered adapter.
        """
        adapter_name = self._get_adapter_name(adapter_name_or_class)
        if adapter_name in self.adapter_dict:
            del(self.adapter_dict[adapter_name])

    def scan(self,
        namespaces: typing.Optional[
            typing.Union[str, typing.List[str]]
        ] = None,
        adapter_classes: typing.Optional[typing.Union[
            typing.Type[BaseAdapter], typing.List[typing.Type[BaseAdapter]]
        ]] = None,
        skip_unknown_namespaces: bool = True,
        recursive_search: bool = True,
        *args, **kwargs
    ):
        """Scan namespaces in search of `adapter_classes` to register.
        """
        if not adapter_classes:
            adapter_classes = [self.adapter_class]
        if type(adapter_classes) is not list:
            adapter_classes = [adapter_classes]

        if not namespaces:
            namespaces = self.namespaces
        if type(namespaces) is not list:
            namespaces = [namespaces]

        loaded_namespaces = sys.modules.keys()
        modules_to_search = []
        for namespace in namespaces:
            if namespace not in loaded_namespaces:
                if recursive_search:
                    modules_to_search.extend([
                        module_name for module_name in loaded_namespaces
                        if module_name.startswith(namespace)
                    ])
                else:
                    if not skip_unknown_namespaces:
                        raise ValueError(
                            f'Unknown namespace `{namespace}` to scan')
            else:
                if recursive_search:
                    modules_to_search.extend([
                        module_name for module_name in loaded_namespaces
                        if module_name.startswith(namespace)
                    ])
                else:
                    modules_to_search.append(namespace)

        found_adapters = []
        for module_name in modules_to_search:
            for adapter_class in adapter_classes:
                module = sys.modules.get(module_name)
                found_adapters.extend(module_classes(module, adapter_class))

        for found_adapter in found_adapters:
            # Skip abstract classes
            if found_adapter.__abstractmethods__:
                continue
            self.register(found_adapter, *args, **kwargs)



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import yaml
import os.path
import pathlib


class IncludeLoader(yaml.SafeLoader):

    """Yaml Loader that allows for !include directive within Yaml file.
    """

    def __init__(self, stream):
        """Input Yaml stream."""
        self._root = os.path.split(stream.name)[0]
        super(IncludeLoader, self).__init__(stream)

    def include(self, node):
        """Take Yaml node and extract filename, then load Yaml file of that
        name."""
        filename = os.path.join(self._root, self.construct_scalar(node))
        filepath = pathlib.Path(filename)
        with filepath.open('r') as file_handle:
            return yaml.load(file_handle, IncludeLoader)


# Make sure we load IncludeLoader on module import
IncludeLoader.add_constructor('!include', IncludeLoader.include)



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import typing
import pathlib
import click

UniversalPath = typing.Union[str, pathlib.Path]
UniversalPathCollection = typing.Iterable[UniversalPath]
CharsList = typing.Union[str, typing.List[str]]
KeywordArgsType = typing.Optional[typing.Dict[str, typing.Any]]
ClickGroupOrCommand = typing.Union[click.Group, click.Command]
AdapterNameOrClass = typing.Union[str, typing.Type['BaseAdapter']]
AdapterManagerNamespaces = typing.Union[str, typing.List[str]]



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

from pkg_resources import get_distribution, require
from email import message_from_string
from webob.multidict import MultiDict


class PackageDef:
    """Grouping of package functions.
    """

    def __init__(self, package_name: str):
        self.package_name = package_name

    @property
    def metadata(self) -> MultiDict:
        """Return package metadata.

        NOTE: Remember to run setup.py develop for this method to work.
        """
        pkgInfo = get_distribution(self.package_name).get_metadata('PKG-INFO')
        msg = message_from_string(pkgInfo)
        metadata = MultiDict(msg)
        return metadata

    @property
    def version(self) -> str:
        """Return package version."""
        __version__ = require(self.package_name)[0].version
        return __version__



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pytest

from ..context import tcutils


DUMMY_CONFIG = {
    'string': 'foo',
    'number': 42,
    'flag': False,
    'nested': {
        'one': 1,
        'two': 2.0,
        'three': 'something',
        'inception': {
            'full': 'stars',
            'nice': 1,
        }
    }
}

CONFIG_ITEMS = [
    ('string', 'foo'),
    ('number', 42),
    ('nested.one', 1),
    ('nested.inception.full', 'stars'),
    ('nested.inception.nice', 1),
]


class TestConfig:

    @pytest.fixture
    def dummy_configuration(self):
        config = tcutils.config.Configuration(DUMMY_CONFIG)
        return config

    def test_dummy_configuration(self, dummy_configuration):
        result = 'Configuration({config_dict})'.format(
            config_dict=str(DUMMY_CONFIG))
        assert str(dummy_configuration) == result

    @pytest.mark.parametrize(
        "item, result", CONFIG_ITEMS
    )
    def test_configuration_attr_access(self, dummy_configuration, item, result):
        item_attr = getattr(dummy_configuration, item)
        assert item_attr == result

    @pytest.mark.parametrize(
        "item, result", CONFIG_ITEMS
    )
    def test_configuration_item_access(self, dummy_configuration, item, result):
        if item.find('.') > -1:
            items = item.split('.')
        else:
            items = [item]
        item_attr = dummy_configuration
        for item_part in items:
            item_attr = item_attr[item_part]
        assert item_attr == result



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pytest

from ..context import tcutils

from .someadapter import SomeAdapter, SomeAdapterManager


class TestAdapters:

    @pytest.fixture
    def some_adapter_manager(self):
        return SomeAdapterManager()

    def test_adapter_manager(self, some_adapter_manager):
        assert some_adapter_manager.name == 'SomeAdapterManager'
        assert some_adapter_manager.namespaces == [
            'tcutils.tests.adapters.test_adapters']
        assert some_adapter_manager.adapter_class == SomeAdapter

    def test_adapter_manager_register_get_remove(self, some_adapter_manager):
        some_adapter_manager.register(SomeAdapter)
        assert SomeAdapter.name in some_adapter_manager.list()
        adapter = some_adapter_manager.get('SomeAdapter')
        assert isinstance(adapter, SomeAdapter)
        some_adapter_manager.remove(SomeAdapter)
        assert SomeAdapter.name not in some_adapter_manager.list()

    def test_adapter_manager_scan(self, some_adapter_manager):
        some_adapter_manager.scan([
            'tcutils.tests.adapters.someadapter',
            'tests.adapters.test_adapters'
        ])
        assert SomeAdapter.name in some_adapter_manager.list()

    def test_adapter_manager_execute(self, some_adapter_manager):
        adapter = SomeAdapter
        some_adapter_manager.register(adapter)
        result = some_adapter_manager.execute(adapter)
        assert result == 'SomeAdapter executing'



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pytest
import types

from ..context import tcutils


class DummyClass:

    somevar = 'one'
    foo = 'bar'

    def __init__(self, other):
        self.something = other

    def one(self):
        pass

    @classmethod
    def cls_method_two(cls, foo):
        pass

    @staticmethod
    def somestatic():
        pass

    def _prefixed_method(self):
        pass

    def _prefixed_other(self):
        pass


CLASS_TYPES_DEFINED = [
    (tcutils.const.CLASS_METHOD_TYPES, ['__init__', 'one', 'cls_method_two',
        'somestatic', '_prefixed_method', '_prefixed_other']),
    ([types.FunctionType], ['__init__', 'one', '_prefixed_method',
        '_prefixed_other']),
    ([classmethod], ['cls_method_two']),
    ([staticmethod], ['somestatic']),
    ([classmethod, staticmethod], ['cls_method_two', 'somestatic']),
]


class TestFuncUtils:

    @pytest.mark.parametrize(
        "method_types, defined", CLASS_TYPES_DEFINED
    )
    def test_class_methods(self, method_types, defined):
        defined = set(defined)
        methods = tcutils.funcutils.class_methods(DummyClass, method_types)
        assert set(methods) == defined

    def test_class_prefixed_methods(self):
        defined = set(['_prefixed_method', '_prefixed_other'])
        methods = tcutils.funcutils.class_prefixed_methods(DummyClass,
            '_prefixed_')
        assert set(methods) == defined



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pytest
import pathlib
import sys

from ..context import tcutils


current_user_name = tcutils.user.current_user_name()

WINDOWS_PATHS = [
    ("Users/someuser", "Users\\someuser"),
    ("C:\\Users\\someuser", "C:\\Users\\someuser"),
    ("$WINDIR", "C:\\Windows"),
    ("$HOMEPATH", "C:\\Users\\" + current_user_name),
    ("%HOMEPATH%", "C:\\Users\\" + current_user_name),
    ("$HOMEPATH/AppData/Local/../Roaming/Python",
        "C:\\Users\\" + current_user_name + "\\AppData\\Roaming\\Python"),
]

WINDOWS_PATHS_EXIST = [
    ("%HOMEPATH%", True),
    ("C:\\NotExistingDirectoryOne", False),
]

WINDOWS_PATHS_JOINED = [
    ("\\Users", "\\Windows", "C:\\Windows"),
    ("\\Users", "%HOMEPATH%", "C:\\Users\\Users\\" + current_user_name),
    ("Windows", "%HOMEPATH%", "Windows\\Users\\" + current_user_name),
    ("%HOMEPATH%", "\\Windows", "C:\\Windows"),
    ("%WINDIR%\\System32", "..\\System", "C:\\Windows\\System"),
]

WINDOWS_PATHS_CREATE = [
    (("tcutils-tests",), "%TEMP%", False),
    (("tcutils-tests/one", "tcutils-tests/two", "tcutils-tests/three"), "%TEMP%", False),
    (("tcutils-tests-2/one/two", "tcutils-tests/one/three",), "%TEMP%", False),
    (("tcutils-tests/two/one", "tcutils-tests/one/three",),
    "%TEMP%\\tcutils-tests-3", True),
]

WINDOWS_CHECK_FILE_INCREMENT = [
    ("%WINDIR%\\System32\\drivers\\etc\\hosts", "C:\\Windows\\System32\\drivers\\etc\\hosts_1"),
]


@pytest.mark.skipif(sys.platform != "win32", reason="does not run on posix")
class TestWindowsPaths:

    @pytest.fixture
    def default_expandvars(self):
        return pathlib.ntpath.expandvars

    @pytest.fixture
    def path_type(self):
        return pathlib.WindowsPath

    @pytest.mark.parametrize(
        "path, result", WINDOWS_PATHS
    )
    @pytest.mark.parametrize(
        "use_default_expansion", [True, False]
    )
    def test_normalize_path(self, path, result, path_type,
        use_default_expansion, default_expandvars
    ):
        normalized_path = tcutils.paths.normalize_path(
            path, use_default_expansion,
            default_expandvars)
        assert normalized_path == path_type(result)

    @pytest.mark.parametrize(
        "path, result", WINDOWS_PATHS
    )
    @pytest.mark.parametrize(
        "use_default_expansion", [True, False]
    )
    def test_get_path(self, path, result, path_type,
        use_default_expansion, default_expandvars
    ):
        normalized_path = tcutils.paths.get_path(
            path, use_default_expansion=use_default_expansion,
            default_expandvars=default_expandvars)
        assert normalized_path == path_type(result)

    @pytest.mark.parametrize(
        "path, path_exists", WINDOWS_PATHS_EXIST
    )
    @pytest.mark.parametrize(
        "use_default_expansion", [True, False]
    )
    def test_check_path(self, path, path_exists,
        use_default_expansion, default_expandvars
    ):
        try:
            result = tcutils.paths.check_path(path,
            use_default_expansion=use_default_expansion,
            default_expandvars=default_expandvars)
        except IOError as e:
            if not path_exists:
                return True
            else:
                raise e

    @pytest.mark.parametrize(
        "path_one, path_two, result", WINDOWS_PATHS_JOINED
    )
    @pytest.mark.parametrize(
        "use_default_expansion", [True, False]
    )
    def test_join_paths(self, path_one, path_two, result, path_type,
        use_default_expansion, default_expandvars
    ):
        joined_path = tcutils.paths.join_paths(path_one, path_two,
            use_default_expansion=use_default_expansion,
            default_expandvars=default_expandvars)

        assert joined_path == path_type(result)


    @pytest.mark.parametrize(
        "use_default_expansion", [True, False]
    )
    def test_current_dir(self, use_default_expansion, default_expandvars):
        current_dir_path = tcutils.paths.current_dir(
            use_default_expansion=use_default_expansion,
            default_expandvars=default_expandvars)
        assert pathlib.Path.cwd() == current_dir_path


    @pytest.mark.parametrize(
        "dirs_to_create, parent_dir, create_parent", WINDOWS_PATHS_CREATE
    )
    @pytest.mark.parametrize(
        "use_default_expansion", [True, False]
    )
    def test_create_dirs(self, dirs_to_create, parent_dir, create_parent,
        use_default_expansion, default_expandvars
    ):
        created_paths = tcutils.paths.create_dirs(dirs_to_create,
            parent_dir, create_parent,
            use_default_expansion=use_default_expansion,
            default_expandvars=default_expandvars)
        parent_path = tcutils.paths.check_path(parent_dir)
        result_paths = [parent_path / x for x in dirs_to_create]
        if create_parent:
            result_paths.insert(0, parent_path)
        try:
            assert created_paths == result_paths
        finally:
            result_paths.sort()
            # Cleanup
            for result_path in result_paths:
                try:
                    result_path.rmdir()
                except IOError:
                    pass


    @pytest.mark.parametrize(
        "filepath, result", WINDOWS_CHECK_FILE_INCREMENT
    )
    @pytest.mark.parametrize(
        "use_default_expansion", [True, False]
    )
    def test_check_file_exists_or_increment(self, filepath, result,
        use_default_expansion, default_expandvars
    ):
        new_filepath = tcutils.paths.check_file_exists_or_increment(filepath,
            use_default_expansion=use_default_expansion,
            default_expandvars=default_expandvars)
        assert new_filepath == pathlib.WindowsPath(result)

    def test_temp_system_dir_path(self):
        temp_dir = tcutils.paths.temp_system_dir_path()
        assert normalize_path('%TEMP%') == temp_dir



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pathlib
from marshmallow import fields, ValidationError

from tcutils.fs import PosixPermissions


class PathField(fields.Field):
    """Field that serializes Path to string and deserializes to pathlib.Path.
    """

    def _serialize(self, value, attr, obj, **kwargs):
        if value is None:
            return ""
        return str(value)

    def _deserialize(self, value, attr, data, **kwargs):
        try:
            return pathlib.Path(value)
        except Exception as e:
            raise ValidationError(f'{e}')


class PosixPermissionsField(fields.Field):
    """Field that serializes PosixPermissions to string and deserializes
    to PosixPermissions.
    """

    def _serialize(self, value, attr, obj, **kwargs):
        if value is None:
            return ""
        return str(value)

    def _deserialize(self, value, attr, data, **kwargs):
        try:
            return PosixPermissions.from_octal(value)
        except Exception as e:
            raise ValidationError(f'{e}')



# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (c) 2019-2020 TropiCoders Karol Tomala
#

import pytest
import getpass

from ..context import tcutils


class TestUser:

    def test_current_user_name(self):
        current_user = tcutils.user.current_user_name()
        assert current_user == getpass.getuser()
